#include "MainWindow.h"
#include "ThreadedLoadImage.h"
#include "RenderViewManager.h"
#include "Scene.h"

#include <QSplitter>

#include <ctkSliderWidget.h>
#include <ctkVTKSliceView.h>
#include <ctkVTKDataSetArrayComboBox.h>
#include <ctkVTKRenderView.h>
#include <ctkFileDialog.h>

#include <ctkPluginFrameworkLauncher.h>
#include <ctkPluginFrameworkFactory.h>
#include <ctkPluginFramework.h>
#include <ctkPluginException.h>
#include <ctkPluginContext.h>
#include <service/event/ctkEventAdmin.h>


#include <vtkActor.h>
#include <vtkAxesActor.h>
#include <vtkImageViewer.h>
#include <vtkInteractorStyleImage.h>
#include <vtkImageProperty.h>
#include <vtkImageSlice.h>
#include <vtkPolyDataMapper.h>
#include <vtkPlane.h>
#include <vtkProperty.h>
#include <vtkRenderWindowInteractor.h>
#include <vtkRenderer.h>
#include <vtkSmartPointer.h>
#include <vtkSphereSource.h>
#include <vtkTransform.h>
#include <vtkRendererCollection.h>
#include <vtkCamera.h>
#include <vtkLightBoxRendererManager.h>
#include <vtkImageResliceMapper.h>

MainWindow::MainWindow(QWidget* parent)
: QMainWindow(parent) 
{	
	InitScene();
	SetupGui();
	//d_scene->importFromFile("X:/Datos/Endoscopy/20120704 Clinic/example VAMSRI/Reconstruccin final/VolumesAndModels.xml");
	InitPluginSystem();
}



void 
MainWindow::InitScene()
{
	d_scene = new Scene(0);
}


void
MainWindow::SetupGui()
{
	RenderViewManager* viewManager = new RenderViewManager(*d_scene);		
	QObject::connect(d_scene,SIGNAL(modelReaded(SurfaceModel*)),viewManager,SLOT(addModel(SurfaceModel*)));
	QObject::connect(d_scene,SIGNAL(volumeReaded(VolumeData*)),viewManager,SLOT(setImageData(VolumeData*)));

	QHBoxLayout* hLayout = new QHBoxLayout();
	hLayout->addWidget(viewManager,2);	
	QGroupBox *sceneView = new QGroupBox("Scene");
	QTreeView* tblView = new QTreeView();
	tblView->setModel(d_scene);
	QObject::connect(tblView,SIGNAL(doubleClicked(const QModelIndex &)),d_scene,SLOT(toggleModelVisibility(const QModelIndex &)));
	hLayout->addWidget(tblView);

	QVBoxLayout* vLayout = new QVBoxLayout();
	vLayout->addLayout(hLayout);
	QStatusBar* statusbar = new QStatusBar();
	statusbar->showMessage("Ready to Work");
	vLayout->addWidget(statusbar);
	
	QWidget* centralWidget = new QWidget();
	setWindowTitle(QApplication::translate("CTKTest", "CTKTest"));
	setCentralWidget(centralWidget);
	centralWidget->setLayout(vLayout);

	SetupMenu();
}

void
MainWindow::SetupMenu()
{
	// SETUP MENU BAR!!!!!
	QMenuBar* mainMenu = menuBar();
	QMenu* sceneMenu;
	sceneMenu = menuBar()->addMenu(QApplication::tr("&Scene"));
	QAction* loadSceneAction = new QAction(QApplication::tr("&Import Scene"),this);	
	sceneMenu->addAction(loadSceneAction);
	QAction* addVolAction = new QAction(QApplication::tr("Add &Volume"),this);	
	sceneMenu->addAction(addVolAction);
	QAction* addModelAction = new QAction(QApplication::tr("Add &Model"),this);
	sceneMenu->addAction(addModelAction);
	sceneMenu->addSeparator();
	QAction* exitAction = new QAction(QApplication::tr("E&xit"),this);
	sceneMenu->addAction(exitAction);
	exitAction->setShortcut(tr("CTRL+E"));
	QMenu* calibrationMenu;
	calibrationMenu = menuBar()->addMenu(QApplication::tr("&Calibration"));
	QAction* calibration1Action = new QAction(QApplication::tr("Calibration &1"),this);
	calibrationMenu->addAction(calibration1Action);
	QAction* calibration2Action = new QAction(QApplication::tr("Calibration &2"),this);
	calibrationMenu->addAction(calibration2Action);
	QMenu* aboutMenu;
	aboutMenu = menuBar()->addMenu(QApplication::tr("&Help"));
	QAction* aboutAction = new QAction(QApplication::tr("&About"),this);
	aboutMenu->addAction(aboutAction);

	ctkFileDialog* fileDialogScene = new ctkFileDialog();
	fileDialogScene->setDirectory("X:/Datos/Endoscopy/20120704 Clinic/example VAMSRI/Reconstruccin final");
	QStringList filters;
	filters << "Scene files (*.xml)" << "Any files (*)";
	fileDialogScene->setNameFilters(filters);

	QObject::connect(loadSceneAction,SIGNAL(triggered()),fileDialogScene,SLOT(exec()));
	QObject::connect(fileDialogScene,SIGNAL(fileSelected(const QString &)),d_scene, SLOT(importFromFile(const QString &)));

	ctkFileDialog* fileDialogVolume = new ctkFileDialog();
	ThreadedLoadImage* loader = new ThreadedLoadImage();
	QObject::connect(addVolAction,SIGNAL(triggered()),fileDialogVolume,SLOT(exec()));
	//QObject::connect(fileDialogVolume,SIGNAL(fileSelected(const QString &)),loader, SLOT(loadImage(const QString&)));

	ctkFileDialog* fileDialogModel = new ctkFileDialog();	
	QObject::connect(addModelAction,SIGNAL(triggered()),fileDialogModel,SLOT(exec()));
	//QObject::connect(fileDialogModel,SIGNAL(fileSelected(const QString &)),loader, SLOT(loadImage(const QString&)));

	QObject::connect(exitAction, SIGNAL(triggered()), this, SLOT(close()));
	
}

void
MainWindow::InitPluginSystem() {
		
	
	//ctkPluginFrameworkLauncher::addSearchPath("C:\\work\\pruebas-bin\\CTKTest\\Plugins\\com.alma.rd.tracker");
	//ctkPluginFrameworkLauncher::addSearchPath("C:\\work\\pruebas-bin\\CTKTest\\Plugins\\com.alma.rd.tracker\\Release");
	ctkPluginFrameworkLauncher::addSearchPath("C:/work/pruebas-bin/CTKTest/Plugins/com.alma.rd.tracker/Release");
	
	bool succeeded = false;
	try {
		succeeded = ctkPluginFrameworkLauncher::start("com.alma.rd.tracker");
	}catch(ctkPluginException &e) {
		std::cout << e.message().toStdString() << std::endl;
		const ctkException* e2 = e.cause();
		if(e2)
			std::cout << e2->message().toStdString() << std::endl;
		
	}catch(ctkRuntimeException &e) {
		std::cout << e.what() << std::endl;
		const ctkException* e2 = e.cause();
		if(e2)
			std::cout << e2->message().toStdString() << std::endl;
	}

	/*
	succeeded = ctkPluginFrameworkLauncher::start("org.commontk.eventadmin");		
	if (succeeded) {
		std::cout << "org.commontk.eventadmin started succesfully" << std::endl;				
		ctkPluginContext* pluginContext = ctkPluginFrameworkLauncher::getPluginContext();
		ctkServiceReference eventAdminRef = pluginContext->getServiceReference<ctkEventAdmin>();
		ctkEventAdmin* eventAdmin = pluginContext->getService<ctkEventAdmin>(eventAdminRef);
		eventAdmin->sendEvent(ctkEvent("mydomain/tutorial/CustomEvent"));		
	} else 
		std::cout << "org.commontk.eventadmin not started" << std::endl;
	
	succeeded = ctkPluginFrameworkLauncher::start("com.alma.rd.trackerlistener");
	if (succeeded) {
		std::cout << "com.alma.rd.trackerlistenerservice started succesfully" << std::endl;		
	} else	
		std::cout << "com.alma.rd.trackerlistenerservice not started" << std::endl;
	*/	
}